(* SYSTEM.def a minimal SYSTEM module.

Copyright (C) 2001-2026 Free Software Foundation, Inc.
Contributed by Gaius Mulley <gaius.mulley@southwales.ac.uk>.

This file is part of GNU Modula-2.

GNU Modula-2 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

GNU Modula-2 is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

Under Section 7 of GPL version 3, you are granted additional
permissions described in the GCC Runtime Library Exception, version
3.1, as published by the Free Software Foundation.

You should have received a copy of the GNU General Public License and
a copy of the GCC Runtime Library Exception along with this program;
see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
<http://www.gnu.org/licenses/>.  *)

DEFINITION MODULE SYSTEM ;

EXPORT QUALIFIED BITSPERBYTE, BYTESPERWORD,
                 LOC, WORD, BYTE, ADDRESS, INTEGER8,  
                 INTEGER16, INTEGER32, INTEGER64, CARDINAL8,  
                 CARDINAL16, CARDINAL32, CARDINAL64, WORD16,  
                 WORD32, WORD64, BITSET8, BITSET16,  
                 BITSET32, REAL32, REAL64, REAL128,  
                 COMPLEX32, COMPLEX64, COMPLEX128, CSIZE_T,  
                 CSSIZE_T, COFF_T,  
                 ADR, TSIZE ;
                 (* SIZE is also exported if -fpim2 is used *)

CONST
  BITSPERBYTE   = __ATTRIBUTE__ __BUILTIN__ ((BITS_PER_UNIT)) ;
  BYTESPERWORD  = __ATTRIBUTE__ __BUILTIN__ ((UNITS_PER_WORD)) ;


(* all the following types are declared internally to gm2
TYPE
   LOC ; 
   WORD ; 
   BYTE ; 
   ADDRESS ; 
   INTEGER8 ; 
   INTEGER16 ; 
   INTEGER32 ; 
   INTEGER64 ; 
   CARDINAL8 ; 
   CARDINAL16 ; 
   CARDINAL32 ; 
   CARDINAL64 ; 
   WORD16 ; 
   WORD32 ; 
   WORD64 ; 
   BITSET8 ; 
   BITSET16 ; 
   BITSET32 ; 
   REAL32 ; 
   REAL64 ; 
   REAL128 ; 
   COMPLEX32 ; 
   COMPLEX64 ; 
   COMPLEX128 ; 
   CSIZE_T ; 
   CSSIZE_T ; 
   COFF_T ; 
*)


END SYSTEM.
